package de.intevation.mxd.reader;

import java.awt.Color;

import org.apache.log4j.Logger;

import com.esri.arcgis.display.ISymbol;
import com.esri.arcgis.display.ILineSymbol;
import com.esri.arcgis.display.HashLineSymbol;
import com.esri.arcgis.display.IRgbColor;
import com.esri.arcgis.display.RgbColor;

import com.esri.arcgis.display.esriLineCapStyle;
import com.esri.arcgis.display.esriLineJoinStyle;

import org.w3c.dom.Element;

/**
 * Reads cartoline symbol information.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class HashLineSymbolReader
extends AbstractSymbolReader {

    /**
     * The logger.
     */
    private static final Logger logger =
        Logger.getLogger(HashLineSymbolReader.class);

    /**
     * Private member.
     */
    private HashLineSymbol symbol;

    public HashLineSymbolReader(ISymbol symbol)
    throws Exception {
        logger.debug("contructor()");
        if(symbol instanceof HashLineSymbol) {
            this.symbol = (HashLineSymbol)symbol;
        }
        else {
            throw new Exception("Not a HashLineSymbol!");
        }
    }

    public HashLineSymbolReader(ILineSymbol symbol)
    throws Exception {
        logger.debug("contructor()");
        if(symbol instanceof HashLineSymbol) {
            this.symbol = (HashLineSymbol)symbol;
        }
        else {
            throw new Exception("Not a HashLineSymbol!");
        }
    }

    /**
     * Reads the symbol attributes.
     *
     * @return The XML node.
     */
    public Element read()
    throws Exception {
        logger.debug("read()");
        Element symbolElement = util.addSymbol(parent);

        symbolElement.setAttribute("name", symbol.getNameString());
        symbolElement.setAttribute("style", "line");
        if(symbol.getColor() instanceof IRgbColor) {
            IRgbColor color = (IRgbColor)symbol.getColor();
            Color c = new Color (
                color.getRed(),
                color.getGreen(),
                color.getBlue());
            symbolElement.setAttribute("color", String.valueOf(c.getRGB()));
        }
        else {
            RgbColor col = new RgbColor();
            col.setRGB(symbol.getColor().getRGB());
            Color c = new Color (
                col.getRed(),
                col.getGreen(),
                col.getBlue());
            symbolElement.setAttribute("color", String.valueOf(c.getRGB()));
        }
        int cap = symbol.getCap();
        String capStyle = "butt";
        switch(cap) {
            case esriLineCapStyle.esriLCSRound: capStyle = "round"; break;
            case esriLineCapStyle.esriLCSSquare: capStyle = "square"; break;
            default: break;
        }
        int join = symbol.getJoin();
        String joinStyle = "bevel";
        switch(join) {
            case esriLineJoinStyle.esriLJSRound: joinStyle = "round"; break;
            case esriLineJoinStyle.esriLJSMitre: joinStyle = "miter"; break;
            default: break;
        }
        symbolElement.setAttribute("cap", capStyle);
        symbolElement.setAttribute("join", joinStyle);
        symbolElement.setAttribute(
            "linestart",
            String.valueOf(symbol.getLineStartOffset()));
        symbolElement.setAttribute(
            "miterlimit",
            String.valueOf(symbol.getMiterLimit()));
        symbolElement.setAttribute(
            "offset",
            String.valueOf(symbol.getOffset()));
        symbolElement.setAttribute("width", String.valueOf(symbol.getWidth()));
        ILineSymbol ls = symbol.getHashSymbol();
        readHashSymbol(ls, symbolElement);

        //TODO Read further HashLine specific attributes:
        //     LineDecoration, Template.

        return symbolElement;
    }

    private void readHashSymbol(ILineSymbol ls, Element parent) 
    throws Exception {
        LineSymbolReader lsr = new LineSymbolReader ();
        if (lsr.canRead(ls)) {
            lsr.setSymbol(ls);
            lsr.setParent(parent);
            lsr.setUtil(util);
            lsr.read();
        }
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
